<?php

namespace FiloBlu\OrderFlowRectifier\Observer;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;

/**
 * Observer on Adyen notification to get cc_type value and update it in table sales_order_payment if missing
 * or fix it if payment is made by Klarna via Adyen
 * This observer needs the adyen module to be patched at least with adyen_notification_events_4303_update.patch
 */

class UpdateCCType implements ObserverInterface
{
    const SCHEDULED_FIX_CCTYPE_ENABLED_CONFIG_PATH = 'order_flow_rectifier/general/scheduled_cctype_check_enabled';
    const PAYMENT_METHOD_ADYEN_CC = 'adyen_cc';
    const PAYMENT_METHOD_ADYEN_HPP = 'adyen_hpp';
    const KLARNA_PAY_LATER = 'klarna_paylater';
    const CC_TYPE_MAP = array(
        "amex" => "AE",
        "visa" => "VI",
        "mc" => "MC",
        "discover" => "DI",
        "jcb" => "JCB",
        "diners" => "DN",
        "maestro" => "MI",
        "cup" => "UN",
        "bcmc" => "BCMC",
        "hipercard" => "HIPERCARD",
        "elo" => "ELO",
        "troy" => "TROY",
        "dankort" => "DANKORT"
    );
    const COUNTRY_KLARNA_PAY_OVER_TIME_MAP = array(
        'IT' => 'klarna_payinthree',
        'ES' => 'klarna_payinthree',
        'FR' => 'klarna_payinthree',
        'NL' => 'klarna_payinthree',
        'GB' => 'klarna_payinthree',
        'PT' => 'klarna_payinthree',
        'IE' => 'klarna_payinthree',
        'DE' => 'klarna_financing',
        'AT' => 'klarna_financing',
        'FI' => 'klarna_financing'
    );

    /**
     * @var ScopeConfigInterface
     */
    protected $config;

    public function __construct(
        ScopeConfigInterface $config
    )
    {
        $this->config = $config;
    }

    /**
     * @param Observer $observer
     */
    public function execute(Observer $observer)
    {
        if ($this->config->isSetFlag(self::SCHEDULED_FIX_CCTYPE_ENABLED_CONFIG_PATH))
        {
            $adyenPaymentMethod = $observer->getEvent()->getPaymentMethod();
            $order = $observer->getEvent()->getOrder();
            $orderPayment = $order->getPayment();
            $paymentMethod = $orderPayment->getMethod();

            if($paymentMethod == self::PAYMENT_METHOD_ADYEN_CC && is_null($orderPayment->getCcType()))
            {
                if(array_key_exists($adyenPaymentMethod, self::CC_TYPE_MAP))
                {
                    $orderPayment->setCcType(self::CC_TYPE_MAP[$adyenPaymentMethod]);
                }
            }
            elseif ($paymentMethod == self::PAYMENT_METHOD_ADYEN_HPP && in_array($adyenPaymentMethod, ['klarna', 'klarna_account']))
            {
                $ccTypeToSet = null;
                if($adyenPaymentMethod == 'klarna')
                {
                    $ccTypeToSet = self::KLARNA_PAY_LATER;
                }
                elseif($adyenPaymentMethod == 'klarna_account')
                {
                    $countryId = $order->getBillingAddress()->getCountryId();
                    if (isset(self::COUNTRY_KLARNA_PAY_OVER_TIME_MAP[$countryId]))
                    {
                        $ccTypeToSet = self::COUNTRY_KLARNA_PAY_OVER_TIME_MAP[$countryId];
                    }
                }
                $orderPayment->setCcType($ccTypeToSet);
            }
        }

        return TRUE;
    }
}